/* ------------------------------------------------------------------------------------------------ */
/* FILE :		 Demo_7_TMX320_P28027_CCS.c															*/
/*																									*/
/* CONTENTS :																						*/
/*				Demo for the Olimex TMX320-P28027 evaluation board (USB serial port)				*/
/*				The trimmer connected on the ADC is use to 											*/
/*				Tasks info & configuration through the serial port									*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2014, Code-Time Technologies Inc. All rights reserved.								*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.1 $																				*/
/*	$Date: 2012/06/17 14:58:30 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 7
	#error "Wrong OS_DEMO value" 
#endif

#include <ctype.h>
#include "Abassi.h"

/* ------------------------------------------------ */

#include "DSP28x_Project.h"
#include <file.h>

#define BAUDRATE	   115200						/* Serial port baud rate						*/
#define SYS_FREQ	60000000L						/* Processor clock frequency					*/

int ___putchar(int);
int ___getchar(void);

void   ADChandler(void);								/* Interrupt handler for the ADC				*/
void   Fct(void);
TSK_t *GetHeadPrio(int Prio);
int    GetKey(void);
void   IOinit(void);
void   UARTinit(void);
void   MemCopy(Uint16 *Src, Uint16 *End, Uint16 *Dst);



/* ------------------------------------------------ */

TSK_t *A_E;											/* A&E knws something in mailbox when event set	*/
MBX_t *MbxADC;										/* Mailbox the ISR put the ADC value read		*/
MBX_t *MbxTime;										/* Mailbox the RTOS timer tick put the RTC value*/

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

main()
{
unsigned int ADCval;
intptr_t     Msg;
unsigned int Time;
int          Tdigits[4];


/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	InitSysCtrl();
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);
	InitFlash();
	InitSciaGpio();
	InitCpuTimers();

/* ------------------------------------------------ */
/* UART set-up										*/

	UARTinit();
	IOinit();

/* ------------------------------------------------ */
/* 	Start the RTOS									*/

	OSstart();

	A_E = TSKmyID();

/* ------------------------------------------------ */
/* SysTick Set-up									*/

/* ------------------------------------------------ */
/* SysTick Set-up (Timer 2)							*/

	OSmapISR(14, 0, TIMtick);						/* Use the RTOS timer for Abassi, vect ID 14	*/

	ConfigCpuTimer(&CpuTimer2, 1.0,(float)(((SYS_FREQ/100000)*(OS_TIMER_US))/(10)));
	CpuTimer2Regs.TCR.all         = 0x4001;
	IER                          |= M_INT14;		/* Timer 2 is attached to INT #14				*/

/* ------------------------------------------------ */
/* ADC set-up										*/

	OSmapISR(32, 1, &ADChandler);					/* ADCINT1is vector ID 32						*/

	InitAdc();


	EALLOW;
	AdcRegs.ADCCTL1.bit.INTPULSEPOS	= 1;			/* ADCINT1 trips after AdcResults latch			*/
	AdcRegs.INTSEL1N2.bit.INT1E     = 1;			/* Enabled ADCINT1								*/
	AdcRegs.INTSEL1N2.bit.INT1CONT  = 0;			/* Disable ADCINT1 Continuous mode				*/
	AdcRegs.INTSEL1N2.bit.INT1SEL	= 2;			/* EOC2 to trigger ADCINT1 to fire				*/
	AdcRegs.ADCSOC0CTL.bit.CHSEL 	= 15;			/* SOC0 channel select to ADCINB7				*/
	AdcRegs.ADCSOC1CTL.bit.CHSEL 	= 15;			/* SOC1 channel select to ADCINB7				*/
	AdcRegs.ADCSOC2CTL.bit.CHSEL 	= 2;			/* SOC2 channel select to ADCINA2				*/
	AdcRegs.ADCSOC0CTL.bit.TRIGSEL 	= 5;			/* SOC0 trig on EPWM1A							*/
	AdcRegs.ADCSOC1CTL.bit.TRIGSEL 	= 5;			/* SOC1 trig on EPWM1A, (RR: after SOC0)		*/
	AdcRegs.ADCSOC2CTL.bit.TRIGSEL 	= 5;			/* SOC2 trig on EPWM1A, (RR: after SOC1)		*/
	AdcRegs.ADCSOC0CTL.bit.ACQPS 	= 6;			/* SOC0 S/H Window: 7 ADC Clk (6 ACQPS + 1)		*/
	AdcRegs.ADCSOC1CTL.bit.ACQPS 	= 6;			/* SOC1 S/H Window: 7 ADC Clk (6 ACQPS + 1)		*/
	AdcRegs.ADCSOC2CTL.bit.ACQPS 	= 6;			/* SOC2 S/H Window: 7 ADC Clk (6 ACQPS + 1)		*/
	EDIS;

	EPwm1Regs.ETSEL.bit.SOCAEN	    = 1;			/* Enable SOC on A group						*/
	EPwm1Regs.ETSEL.bit.SOCASEL	    = 4;			/* Select SOC from from CPMA on upcount			*/
	EPwm1Regs.ETPS.bit.SOCAPRD 	    = 1;			/* Generate pulse on 1st event					*/
	EPwm1Regs.CMPA.half.CMPA 	    = 0x0080;		/* Set compare A value							*/
	EPwm1Regs.TBPRD 				= 0xFFFF;		/* Set period for ePWM1							*/
	EPwm1Regs.TBCTL.bit.CTRMODE 	= 0;			/* Count up and start							*/

	IER |= M_INT1;									/* ADCINT1 is attached to PIE interrupt group 1	*/
	PieCtrlRegs.PIEIER1.bit.INTx1 = 1;				/* Enable ADCINT1 which is attached to INT1.1	*/


/* ------------------------------------------------ */
/* Application set-up								*/

	MbxADC  = MBXopen("ADC", 5);
	MbxTime = MBXopen("Time", 5);

/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	OSeint(1);
	ERTM;

	puts("\n\n");									/* Welcome banner								*/
	puts("\n       Abassi Demo 7\n");
	puts("\n    Olimex TMX320-28027\n");
	puts("\n      Use the trimmer\n\n");

	Tdigits[0] = 0;
	Tdigits[1] = 0;
	Tdigits[2] = 0;
	Tdigits[3] = 0;

	for (;;) {
		EVTwait(0x0000, 0xFFFF, -1);				/* A&E Waiting for a new time or ADC value		*/
		while (0 == MBXget(MbxTime, &Msg, 0)) {		/* Empty the mailbox with the running time 		*/
			Time = (unsigned int)Msg;				/* The ISR limits the time to < 3600			*/
			Tdigits[0] = Time       / 600;			/* Minutes MS digit								*/
			Time      -= Tdigits[0] * 600; 
			Tdigits[1] = Time       / 60;			/* Minutes LS digit								*/
			Time      -= Tdigits[1] * 60; 
			Tdigits[2] = Time       / 10;			/* Seconds MS digit								*/
			Time      -= Tdigits[2] * 10; 
			Tdigits[3] = Time;						/* Seconds LS digit								*/
		}
			
		while(0 == MBXget(MbxADC, &Msg, 0)) {		/* Always completely mpty the mailbox			*/
			ADCval = (unsigned int)Msg;				/* Memo the ADC value 							*/
		}

		printf("[%d%d:%d%d] ADC:%u%%       \r", Tdigits[0], Tdigits[1], Tdigits[2], Tdigits[3],
		        (int)((double)ADCval/40.95));
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{
	for(;;);										/* Idle is a do nothing task					*/
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* RTOS Timer call back function																	*/
/*																									*/
/* Here we use the macro OS_TICK_PER_SEC to get the XXX led to toggle once every second				*/
/* The LED is not toggle here; instead FctHI gets awaken by posting SemRed							*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void TIMcallBack(void)
{
static int SecCnt  = 0;								/* Conter to incrm Seconds once per second		*/
static int Seconds = 0;								/* RTC cumulative seconds						*/

	SecCnt++;
	if (SecCnt >= OS_TICK_PER_SEC) {				/* Time to add one more second					*/
		Seconds++;
		if (Seconds >= 3600) {						/* The maximum display value is 59:59			*/
			Seconds = 0;
		}
		SecCnt = 0;
		MBXput(MbxTime, (intptr_t)Seconds, 0);		/* Send to the ADC task the new time in seconds	*/
		EVTset(A_E, 0x1);
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* ADC interrupt handler																			*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void ADChandler(void)
{
       int  ADCint;
static long ADCavg = 0L;
static int  ADCcnt = 0;


	ADCavg += (long)AdcResult.ADCRESULT1;			/* Read the ADC value and add to average		*/

	ADCcnt++;
	if (ADCcnt >= 64) {								/* Averaging 64 times							*/
		ADCint = (int)((ADCavg+32L) >> 6);			/* Rounding and /64								*/
		MBXput(MbxADC, (intptr_t)ADCint, 0);		/* Send ADC value to the A&E through a mailbox	*/
		EVTset(A_E, 0x1);							/* Inform A&E there is someting in one mailbox	*/
		ADCavg = 0L;								/* Restart the averaging						*/
		ADCcnt = 0;
	}

	IER |= M_INT1;									/* Re-enable INT1								*/
	AdcRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;			/* Clear ADCINT1 flag reinitialize for next SOC	*/
	PieCtrlRegs.PIEACK.bit.ACK1 = 0x1;				/* ADCINT1 is in PIE group 1, acknowledge it	*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void MemCopy(Uint16 *Src, Uint16 *End, Uint16 *Dst)
{
	while (Src < End) {
		*Dst++ = *Src++;
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UARTinit(void)
{

    SciaRegs.SCICCR.all             = 0x0007;		/* 1 stop bit,  No loopback, No parity,8 bits,	*/
													/* async mode, idle-line protocol				*/
    SciaRegs.SCICTL1.all            = 0x0003;		/* enable TX, RX, internal SCICLK,				*/
													/* Disable RX ERR, SLEEP, TXWAKE				*/
    SciaRegs.SCICTL2.all            = 0x0000;
    SciaRegs.SCICTL2.bit.TXINTENA   = 0;			/* No TX interrupt								*/
    SciaRegs.SCICTL2.bit.RXBKINTENA = 0;			/* No RX interrupt								*/
	SciaRegs.SCIHBAUD               = 0x0000;
	SciaRegs.SCILBAUD               = ((SYS_FREQ)/(BAUDRATE))/32;

    SciaRegs.SCICTL1.all            = 0x0023;		/* Relinquish SCI from Reset					*/		

    SciaRegs.SCIFFTX.all            = 0xE040;		/* Set-up FIFOs									*/
    SciaRegs.SCIFFRX.all            = 0x2044;
    SciaRegs.SCIFFCT.all            = 0x0;

	return;
}

/* ------------------------------------------------------------------------------------------------ */

int UARTopen(const char *path, unsigned int flags, int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTclose(int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTread(int fd, char *buf, unsigned int count)
{
	*buf = ___getchar();
	return(1);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTwrite(int fd, const char *buf, unsigned int count)
{
int ii;

	for(ii=0 ; ii<count ; ii++) {
		 ___putchar(*buf++);
	}
	return(count);
}

/* ------------------------------------------------------------------------------------------------ */

off_t UARTseek(int fd, off_t off, int start)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTunlink(const char *path)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTrename(const char *old, const char *new)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

void IOinit(void) {
	add_device("uart", _MSA, UARTopen, UARTclose, UARTread, UARTwrite, UARTseek, UARTunlink, UARTrename);
	freopen("uart:stdout", "w", stdout);
	setvbuf(stdout, NULL, _IONBF, 0);
	freopen("uart:stdin", "r", stdin);
	setvbuf(stdin, NULL, _IONBF, 0);
}

/* ------------------------------------------------------------------------------------------------ */

int ___putchar(int ccc)
{
	if (ccc == '\n') {
    	for( ; SciaRegs.SCIFFTX.bit.TXFFST != 0 ; );
	    SciaRegs.SCITXBUF = '\r';
	}
   	for( ; SciaRegs.SCIFFTX.bit.TXFFST != 0 ; );
    SciaRegs.SCITXBUF = ccc;

	return(ccc);
}

/* ------------------------------------------------------------------------------------------------ */

int ___getchar(void)
{
int RetVal;

	for ( ; SciaRegs.SCIFFRX.bit.RXFFST == 0 ; );
	RetVal = SciaRegs.SCIRXBUF.all;

	return(RetVal);
}

/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
int RetVal;

	RetVal = 0;

	if (SciaRegs.SCIFFRX.bit.RXFFST != 0) {
		RetVal = SciaRegs.SCIRXBUF.all;
	}

	return(RetVal);
}

/* EOF */
